# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# opensecurity-dialog
# 
# an opensecurity dialog
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright (C) 2013 AIT Austrian Institute of Technology
# AIT Austrian Institute of Technology GmbH
# Donau-City-Strasse 1 | 1220 Vienna | Austria
# http://www.ait.ac.at
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, 
# Boston, MA  02110-1301, USA.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import argparse
import json
import os
import subprocess
import sys
import urllib
import urllib2
import webbrowser

from PyQt4 import QtCore
from PyQt4 import QtGui

# local
import __init__ as opensecurity

if sys.platform == 'win32' or sys.platform == 'cygwin':
    from cygwin import Cygwin

import opensecurity_client_restful_server 
from ui import AboutDialog
from ui import ConfigureDialog
from ui import opensecurity_rc


# ------------------------------------------------------------
# code


class OpenSecurityWait(QtGui.QDialog):

    """OpenSecurity: please wait ..."""
    
    def __init__(self, parent = None, flags = QtCore.Qt.WindowFlags(0)):
        super(OpenSecurityWait, self).__init__(parent, flags)
        self.setWindowTitle('OpenSecurity')
        self.setup_ui()
        
        
    def setup_ui(self):
        """Create the widgets."""
        
        lyMain = QtGui.QVBoxLayout(self)
        lyMain.setContentsMargins(8, 8, 8, 8)
        
        # content area: left pixmap, right text
        lbTitle = QtGui.QLabel('Creating secure subsystem. Please stand by ...')
        lyMain.addWidget(lbTitle)
        
        self.setMinimumSize(400, 50)
        self.resize(lyMain.minimumSize())


class OpenSecurityTrayIcon(QtGui.QSystemTrayIcon):
    
    """This is the OpenSecuirty Tray Icon"""

    def __init__(self, icon, parent=None):
        
        super(OpenSecurityTrayIcon, self).__init__(icon, parent)
        self.setup_ui()
        
        
    def clicked_about(self):
        """clicked about"""
        d = AboutDialog()
        d.exec_()
    

    def clicked_browser(self):
        """wish for safe internet browsing"""
        
        if not (sys.platform == 'win32' or sys.platform == 'cygwin'):
            QtGui.QMessageBox.critical(self.parent(), 'OpenSecurity Error', 'This action is not supported on this platform.\nSorry.')
            return
       
        try:
        
            # get a proper browsing VM
            Cygwin.start_X11()

            # TODO: HARDCODED ADDRESS OF OPENSECURITYD
            browsing_vm = urllib2.urlopen('http://127.0.0.1:8080/browsing').readline()
            print('Called http://127.0.0.1:8080/browsing got: ' + str(browsing_vm))
            
        except:
            
            QtGui.QApplication.instance().processEvents()
            QtGui.QMessageBox.critical(None, 'Failed to invoke Safe Internet Browsing', 'OpenSecurity Error')
            
        QtGui.QApplication.instance().processEvents()
            
            
    def clicked_configure(self):
        """clicked configure"""
        d = ConfigureDialog()
        d.exec_()
    

    def clicked_exit(self):
        """clicked exit"""
        opensecurity_client_restful_server.stop()
        sys.exit(0)
    

    def clicked_launch_application(self):
        """clicked the launch an application"""
        dlg_launch_image = os.path.join(sys.path[0], 'ui', 'launch_dialog.py')
        process_command = [sys.executable, dlg_launch_image]
        process = subprocess.Popen(process_command, shell = False, stdout = subprocess.PIPE)        
        try:
            stdout = process.communicate()[0]
            j = json.loads(stdout)
        except:
            return

        try:
        
            # get a proper browsing VM
            Cygwin.start_X11()

            # TODO: HARDCODED ADDRESS OF OPENSECURITYD
            url = 'http://127.0.0.1:8080/sdvms/' + j['vm'] + '/application' + j['application']
            result = urllib2.urlopen(url).readline()
            
        except:
            pass 
            
            
    def clicked_mail(self):
        
        """clicked mail"""
        address = 'feedback@opensecurity.at'
        subject = 'Feedback zu OpenSecurity V' + opensecurity.__version__

        if sys.platform == 'linux2':
            subprocess.Popen(['xdg-email', '--subject', subject, address])
        elif sys.platform == 'win32' or sys.platform == 'cygwin':
            mail_url = 'mailto:' + urllib.quote(address, '@') + '?' + urllib.quote('subject=' + subject)
            subprocess.Popen(['cmd', '/C', 'start', mail_url])
    

    def setup_ui(self):
        """create the user interface
        As for the system tray this is 'just' the context menu.
        """
    
        # define the tray icon menu
        menu = QtGui.QMenu(self.parent())
        self.setContextMenu(menu)
        
        # add known apps
        self.acBrowser = QtGui.QAction('Secure Browsing', self.parent())
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(QtCore.QString.fromUtf8(':/opensecurity/gfx/opensecurity_browsing_64.png')), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.acBrowser.setIcon(icon)
        menu.addAction(self.acBrowser)
        menu.addSeparator()
        
        # add standard menu items
        self.acLaunch = QtGui.QAction('Launch Application', self.parent())
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(QtCore.QString.fromUtf8(':/opensecurity/gfx/opensecurity_launch_64.png')), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.acLaunch.setIcon(icon)
        menu.addAction(self.acLaunch)
        self.acConfigure = QtGui.QAction('Configuration', self.parent())
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(QtCore.QString.fromUtf8(':/opensecurity/gfx/opensecurity_configure_64.png')), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.acConfigure.setIcon(icon)
        menu.addAction(self.acConfigure)
        menu.addSeparator()

        self.acMail = menu.addAction('Send feedback mail')
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(QtCore.QString.fromUtf8(':/opensecurity/gfx/opensecurity_mail_64.png')), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.acMail.setIcon(icon)
        self.acAbout = menu.addAction('About')
        self.acExit = menu.addAction('Exit')
       
        self.acBrowser.triggered.connect(self.clicked_browser)
        self.acLaunch.triggered.connect(self.clicked_launch_application)
        self.acConfigure.triggered.connect(self.clicked_configure)
        self.acAbout.triggered.connect(self.clicked_about)
        self.acExit.triggered.connect(self.clicked_exit)
        self.acMail.triggered.connect(self.clicked_mail)
        

def main():
    
    # parse arguments
    parser = argparse.ArgumentParser(description = 'OpenSecurity Tray Icon.')
    parser.add_argument('-p', '--port', type=int, default=8090, help='port number of the REST API this instance will listen on.')
    args = parser.parse_args()

    # get up Qt
    a = QtGui.QApplication(sys.argv)

    # enforce singelton process
    if opensecurity_client_restful_server.is_already_running(args.port):
        QtGui.QMessageBox.critical(None, 'OpenSecurity Error', 'OpenSecurity Tray Instance already launched.\nClose previous instance first.')
        sys.exit(1)

    # start serving
    opensecurity_client_restful_server.serve(args.port, True)

    # init tray icon widget
    w = QtGui.QWidget()
    icon = QtGui.QIcon()
    icon.addPixmap(QtGui.QPixmap(QtCore.QString.fromUtf8(":/opensecurity/gfx/opensecurity_icon_64.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
    trayIcon = OpenSecurityTrayIcon(icon)
    opensecurity_client_restful_server.tray_icon = trayIcon

    # go!
    trayIcon.show()
    a.setQuitOnLastWindowClosed(False)
    sys.exit(a.exec_())
   

# start
if __name__ == "__main__":
    main()

